package ch.std.console.context;

import java.io.IOException;
import java.lang.annotation.Annotation;
import java.util.Locale;
import java.util.Map;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.beans.factory.config.AutowireCapableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.MessageSourceResolvable;
import org.springframework.context.NoSuchMessageException;
import org.springframework.core.ResolvableType;
import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;

public class CustomAppContext implements ApplicationContext {

    private ApplicationContext applicationContext;
    private RepositoryContext repositoryContext;

    public static CustomAppContext getCurrentContext() throws Exception {
    	CustomAppContext currentContext = ApplicationContextProvider.getContext();
        if (currentContext == null) {
            currentContext = new CustomAppContext(null);
            ApplicationContextProvider.setContext(currentContext);
        }
        return currentContext;
    }

    public CustomAppContext(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
        this.repositoryContext = new RepositoryContext();
    }

    public RepositoryContext getRepositoryContext() {
        return this.repositoryContext;
    }

    /**
     * @see ApplicationContext#getId()
     *
     * @return
     */
    @Override
    public String getId() {
        return this.applicationContext.getId();
    }

    /**
     * @see ApplicationContext#getApplicationName()
     *
     * @return
     */
    @Override
    public String getApplicationName() {
        return this.applicationContext.getApplicationName();
    }

    /**
     * @see ApplicationContext#getDisplayName()
     *
     * @return
     */
    @Override
    public String getDisplayName() {
        return this.applicationContext.getDisplayName();
    }

    /**
     * @see ApplicationContext#getStartupDate()
     *
     * @return
     */
    @Override
    public long getStartupDate() {
        return this.applicationContext.getStartupDate();
    }

    /**
     * @see ApplicationContext#getParent()
     *
     * @return
     */
    @Override
    public ApplicationContext getParent() {
        return this.applicationContext.getParent();
    }

    /**
     * @see ApplicationContext#getAutowireCapableBeanFactory()
     *
     * @return
     */
    @Override
    public AutowireCapableBeanFactory getAutowireCapableBeanFactory() throws IllegalStateException {
        return this.applicationContext.getAutowireCapableBeanFactory();
    }

    /**
     * @see ApplicationContext#getParentBeanFactory()
     *
     * @return
     */
    @Override
    public BeanFactory getParentBeanFactory() {
        return this.applicationContext.getParentBeanFactory();
    }

    /**
     * @see ApplicationContext#containsLocalBean(String)
     *
     * @return
     */
    @Override
    public boolean containsLocalBean(String s) {
        return this.applicationContext.containsLocalBean(s);
    }

    /**
     * @see ApplicationContext#containsBeanDefinition(String)
     *
     * @return
     */
    @Override
    public boolean containsBeanDefinition(String s) {
        return this.applicationContext.containsBeanDefinition(s);
    }

    /**
     * @see ApplicationContext#getBeanDefinitionCount()
     *
     * @return
     */
    @Override
    public int getBeanDefinitionCount() {
        return this.applicationContext.getBeanDefinitionCount();
    }

    /**
     * @see ApplicationContext#getBeanDefinitionNames()
     *
     * @return
     */
    @Override
    public String[] getBeanDefinitionNames() {
        return this.applicationContext.getBeanDefinitionNames();
    }

    /**
     * @see ApplicationContext#getBeanNamesForType(ResolvableType)
     *
     * @return
     */
    @Override
    public String[] getBeanNamesForType(ResolvableType resolvableType) {
        return this.applicationContext.getBeanNamesForType(resolvableType);
    }

    /**
     * @see ApplicationContext#getBeanNamesForType(Class)
     *
     * @return
     */
    @Override
    public String[] getBeanNamesForType(Class<?> aClass) {
        return this.applicationContext.getBeanNamesForType(aClass);
    }

    /**
     * @see ApplicationContext#getBeanNamesForType(Class, boolean, boolean)
     *
     * @return
     */
    @Override
    public String[] getBeanNamesForType(Class<?> aClass, boolean b, boolean b1) {
        return this.applicationContext.getBeanNamesForType(aClass, b, b1);
    }

    /**
     * @see ApplicationContext#getBeansOfType(Class)
     *
     * @return
     */
    @Override
    public <T> Map<String, T> getBeansOfType(Class<T> aClass) throws BeansException {
        return this.applicationContext.getBeansOfType(aClass);
    }

    /**
     * @see ApplicationContext#getBeansOfType(Class, boolean, boolean)
     *
     * @return
     */
    @Override
    public <T> Map<String, T> getBeansOfType(Class<T> aClass, boolean b, boolean b1) throws BeansException {
        return this.applicationContext.getBeansOfType(aClass, b, b1);
    }

    /**
     * @see ApplicationContext#getBeanNamesForAnnotation(Class)
     *
     * @return
     */
    @Override
    public String[] getBeanNamesForAnnotation(Class<? extends Annotation> aClass) {
        return this.applicationContext.getBeanNamesForAnnotation(aClass);
    }

    /**
     * @see ApplicationContext#getBeansWithAnnotation(Class)
     *
     * @return
     */
    @Override
    public Map<String, Object> getBeansWithAnnotation(Class<? extends Annotation> aClass) throws BeansException {
        return this.applicationContext.getBeansWithAnnotation(aClass);
    }

    /**
     * @see ApplicationContext#findAnnotationOnBean(String, Class)
     *
     * @return
     */
    @Override
    public <A extends Annotation> A findAnnotationOnBean(String s, Class<A> aClass) throws NoSuchBeanDefinitionException {
        return this.applicationContext.findAnnotationOnBean(s, aClass);
    }

    /**
     * @see ApplicationContext#getBean(String)
     *
     * @return
     */
    @Override
    public Object getBean(String s) throws BeansException {
        return this.applicationContext.getBean(s);
    }

    /**
     * @see ApplicationContext#getBean(String, Class)
     *
     * @return
     */
    @Override
    public <T> T getBean(String s, Class<T> aClass) throws BeansException {
        return this.applicationContext.getBean(s, aClass);
    }

    /**
     * @see ApplicationContext#getBean(String, Object...)
     *
     * @return
     */
    @Override
    public Object getBean(String s, Object... objects) throws BeansException {
        return this.applicationContext.getBean(s, objects);
    }

    /**
     * @see ApplicationContext#getBean(Class)
     *
     * @return
     */
    @Override
    public <T> T getBean(Class<T> aClass) throws BeansException {
        return this.applicationContext.getBean(aClass);
    }

    /**
     * @see ApplicationContext#getBean(Class, Object...)
     *
     * @return
     */
    @Override
    public <T> T getBean(Class<T> aClass, Object... objects) throws BeansException {
        return this.applicationContext.getBean(aClass, objects);
    }

	@Override
	public <T> ObjectProvider<T> getBeanProvider(Class<T> requiredType) {
		return this.applicationContext.getBeanProvider(requiredType);
	}

	@Override
	public <T> ObjectProvider<T> getBeanProvider(ResolvableType requiredType) {
		return this.applicationContext.getBeanProvider(requiredType);
	}

    /**
     * @see ApplicationContext#containsBean(String)
     *
     * @return
     */
    @Override
    public boolean containsBean(String s) {
        return this.applicationContext.containsBean(s);
    }

    /**
     * @see ApplicationContext#isSingleton(String)
     *
     * @return
     */
    @Override
    public boolean isSingleton(String s) throws NoSuchBeanDefinitionException {
        return this.applicationContext.isSingleton(s);
    }

    /**
     * @see ApplicationContext#isPrototype(String)
     *
     * @return
     */
    @Override
    public boolean isPrototype(String s) throws NoSuchBeanDefinitionException {
        return this.applicationContext.isPrototype(s);
    }

    /**
     * @see ApplicationContext#isTypeMatch(String, ResolvableType)
     *
     * @return
     */
    @Override
    public boolean isTypeMatch(String s, ResolvableType resolvableType) throws NoSuchBeanDefinitionException {
        return this.applicationContext.isTypeMatch(s, resolvableType);
    }

    /**
     * @see ApplicationContext#isTypeMatch(String, ResolvableType)
     *
     * @return
     */
    @Override
    public boolean isTypeMatch(String s, Class<?> aClass) throws NoSuchBeanDefinitionException {
        return this.applicationContext.isTypeMatch(s, aClass);
    }

    /**
     * @see ApplicationContext#getType(String)
     *
     * @return
     */
    @Override
    public Class<?> getType(String s) throws NoSuchBeanDefinitionException {
        return this.applicationContext.getType(s);
    }

    /**
     * @see ApplicationContext#getAliases(String)
     *
     * @return
     */
    @Override
    public String[] getAliases(String s) {
        return this.applicationContext.getAliases(s);
    }

    /**
     * @see ApplicationContext#publishEvent(ApplicationEvent)
     *
     * @return
     */
    @Override
    public void publishEvent(ApplicationEvent applicationEvent) {
        this.applicationContext.publishEvent(applicationEvent);
    }

    /**
     * @see ApplicationContext#publishEvent(Object)
     *
     * @return
     */
    @Override
    public void publishEvent(Object o) {
        this.applicationContext.publishEvent(o);
    }

    /**
     * @see ApplicationContext#getMessage(String, Object[], String, Locale)
     *
     * @return
     */
    @Override
    public String getMessage(String s, Object[] objects, String s1, Locale locale) {
        return this.applicationContext.getMessage(s, objects, s1, locale);
    }

    /**
     * @see ApplicationContext#getMessage(String, Object[], String, Locale)
     *
     * @return
     */
    @Override
    public String getMessage(String s, Object[] objects, Locale locale) throws NoSuchMessageException {
        return this.applicationContext.getMessage(s, objects, locale);
    }

    /**
     * @see ApplicationContext#getMessage(MessageSourceResolvable, Locale)
     *
     * @return
     */
    @Override
    public String getMessage(MessageSourceResolvable messageSourceResolvable, Locale locale) throws NoSuchMessageException {
        return this.applicationContext.getMessage(messageSourceResolvable, locale);
    }

    /**
     * @see ApplicationContext#getResources(String)
     *
     * @return
     */
    @Override
    public Resource[] getResources(String locationPattern) throws IOException {
        return this.applicationContext.getResources(locationPattern);
    }

    /**
     * @see ApplicationContext#getResource(String)
     *
     * @return
     */
    @Override
    public Resource getResource(String location) {
        return this.applicationContext.getResource(location);
    }

    /**
     * @see ApplicationContext#getClassLoader()
     *
     * @return
     */
    @Override
    public ClassLoader getClassLoader() {
        return this.applicationContext.getClassLoader();
    }

    /**
     * see ApplicationContext#getEnvironment()
     */
	@Override
	public Environment getEnvironment() {
		return this.applicationContext.getEnvironment();
	}

	@Override
	public String[] getBeanNamesForType(ResolvableType type, boolean includeNonSingletons, boolean allowEagerInit) {
		return this.applicationContext.getBeanNamesForType(type, includeNonSingletons, allowEagerInit);
	}

	@Override
	public Class<?> getType(String name, boolean allowFactoryBeanInit) throws NoSuchBeanDefinitionException {
		return this.getType(name, allowFactoryBeanInit);
	}

	@Override
	public <T> ObjectProvider<T> getBeanProvider(Class<T> requiredType, boolean allowEagerInit) {
		return this.applicationContext.getBeanProvider(requiredType, allowEagerInit);
	}

	@Override
	public <T> ObjectProvider<T> getBeanProvider(ResolvableType requiredType, boolean allowEagerInit) {
		return this.applicationContext.getBeanProvider(requiredType, allowEagerInit);
	}

	@Override
	public <A extends Annotation> A findAnnotationOnBean(String beanName, Class<A> annotationType,
			boolean allowFactoryBeanInit) throws NoSuchBeanDefinitionException {
		return this.applicationContext.findAnnotationOnBean(beanName, annotationType, allowFactoryBeanInit);
	}

}

